import csv
import taichi as ti
from configparser import ConfigParser
from config import Config
from orca_people import ORCA_People
from sf_people import SF_People
from steer_people import Steer_People
import taichi as ti
import time
import utils

config = Config()
# simulation method, defined in utils.py dictionary. default:social force
simu_method = 1
# output csv file or not, 0 means not output
export_csv = 0
# output file path
out_path = ""

def read_ini_file(N,file_name):
    """read ini file and set config"""
    print('-'*10,N)
    cfg = ConfigParser()
    cfg.read(file_name)
    cfg.set("CONFIG_VALUE","N",str(N))
    cfg.write(open(file_name, "r+"))
    src = cfg.sections() #会过滤掉default
    for section in src:
        if section == "SIMULATION":
            # set simulation config value
            for item in cfg.items(section):
                # if item[0] == "frame":
                #     frame = int(item[1])
                if item[0] == "export_csv":
                    export_csv = int(item[1])
                elif item[0] == "method":
                    # get simulation method from utils.py dictionary, if not found, use ORCA
                    simu_method = utils.METHODS.get(item[1],1) 
                
        elif section == "SCENE":
            # set scene predefined in config.py
            for item in cfg.items(section):
                if item[0] == "scene":
                    config.set_scene(item[1])
                    
        else: 
            # set config value
            for item in cfg.items(section):# items会使得key和value都变为小写str类型
                config.set(item[0], item[1])
    config.post_init()


ti.init(arch=ti.gpu)#, debug=True,cpu_max_num_threads=16
TOTAL_FRAME = 50

with open('outfile.csv', 'w', newline='') as csvfile:
    writer = csv.writer(csvfile)

    #先写入columns_name
    writer.writerow(["people_num","FPS"])
    for N in range (75000,102000,5000):
        # 将inifile的N值改为N
        read_ini_file(N,"longhall_socialforce.ini") 
        #gui = ti.GUI("scene_name", res=(config.WINDOW_WIDTH, config.WINDOW_HEIGHT),background_color=0xffffff)
        frame = 0
        people = None
        if simu_method == 0:
            people = ORCA_People(config)
        elif simu_method == 1:
            people = SF_People(config)
        elif simu_method == 2:
            people = Steer_People(config)

        # Perform (and manipulate) the simulation.
        end = time.time()

        #while gui.running:
        while frame < TOTAL_FRAME:
            frame += 1
            # if gui.get_event(ti.GUI.RMB,(ti.GUI.PRESS,ti.GUI.SPACE)):
            #     if gui.event.key == ti.GUI.SPACE: #空格键退出
            #         gui.running = False       
            people.compute()
            people.update()
            # people.render(gui)
            # gui.show()

        end_run = time.time()
        simulation_time = end_run - end
        print("total time: " + str(simulation_time)+ "s")
        print("total_frame: " + str(frame))
        # FPS = frame per second
        print("FPS: " + str(frame / simulation_time))
        writer.writerow([str(N) ,str(frame / simulation_time)])